@@ -125,7 +125,7 @@ static void     ibus_im_context_set_surrounding
 /* static methods*/
 static void     _create_input_context       (IBusIMContext      *context);
 static gboolean _set_cursor_location_internal
-                                            (GtkIMContext       *context);
+                                            (IBusIMContext      *context);
 
 static void     _bus_connected_cb           (IBusBus            *bus,
                                              IBusIMContext      *context);
@@ -147,8 +147,7 @@ static gboolean _slave_delete_surrounding_cb
                                              gint                offset_from_cursor,
                                              guint               nchars,
                                              IBusIMContext      *context);
-static void     _request_surrounding_text   (IBusIMContext      *context,
-                                             gboolean            force);
+static void     _request_surrounding_text   (IBusIMContext      *context);
 static void     _create_fake_input_context  (void);
 
 
@@ -246,13 +245,18 @@ _process_key_event_done (GObject      *object,
     IBusInputContext *context = (IBusInputContext *)object;
     GdkEventKey *event = (GdkEventKey *) user_data;
 
-    gboolean processed = FALSE;
-    if (!ibus_input_context_process_key_event_async_finish (context,
-            res, &processed, NULL)) {
-        processed = FALSE;
+    GError *error = NULL;
+    gboolean retval = ibus_input_context_process_key_event_async_finish (
+            context,
+            res,
+            &error);
+
+    if (error != NULL) {
+        g_warning ("Process Key Event failed: %s.", error->message);
+        g_error_free (error);
     }
 
-    if (!processed) {
+    if (retval == FALSE) {
         event->state |= IBUS_IGNORED_MASK;
         gdk_event_put ((GdkEvent *)event);
     }
@@ -263,17 +267,13 @@ _process_key_event_done (GObject      *object,
 /* emit "retrieve-surrounding" glib signal of GtkIMContext, if
  * context->caps has IBUS_CAP_SURROUNDING_TEXT and the current IBus
  * engine needs surrounding-text.
- *
- * if "force" is TRUE, emit the signal regardless of whether the
- * engine needs surrounding-text.
  */
 static void
-_request_surrounding_text (IBusIMContext *context, gboolean force)
+_request_surrounding_text (IBusIMContext *context)
 {
-    if (context->enable &&
+    if (context && context->enable &&
         (context->caps & IBUS_CAP_SURROUNDING_TEXT) != 0 &&
-        (force ||
-         ibus_input_context_needs_surrounding_text (context->ibuscontext))) {
+        ibus_input_context_needs_surrounding_text (context->ibuscontext)) {
         gboolean return_value;
         IDEBUG ("requesting surrounding text");
         g_signal_emit (context, _signal_retrieve_surrounding_id, 0,
@@ -368,9 +368,8 @@ _key_snooper_cb (GtkWidget   *widget,
 
     } while (0);
 
-    _request_surrounding_text (ibusimcontext, FALSE);
-
     if (ibusimcontext != NULL) {
+        _request_surrounding_text (ibusimcontext);
         ibusimcontext->time = event->time;
     }
 
@@ -680,7 +679,7 @@ ibus_im_context_filter_keypress (GtkIMContext *context,
         if (ibusimcontext->client_window == NULL && event->window != NULL)
             gtk_im_context_set_client_window ((GtkIMContext *)ibusimcontext, event->window);
 
-        _request_surrounding_text (ibusimcontext, FALSE);
+        _request_surrounding_text (ibusimcontext);
 
         if (ibusimcontext != NULL) {
             ibusimcontext->time = event->time;
@@ -758,12 +757,12 @@ ibus_im_context_focus_in (GtkIMContext *context)
      * it blocks UI. So delay it to idle callback. */
     g_idle_add_full (G_PRIORITY_DEFAULT_IDLE,
                      (GSourceFunc) _set_cursor_location_internal,
-                     g_object_ref (context),
+                     g_object_ref (ibusimcontext),
                      (GDestroyNotify) g_object_unref);
 
     /* retrieve the initial surrounding-text (regardless of whether
      * the current IBus engine needs surrounding-text) */
-    _request_surrounding_text (ibusimcontext, TRUE);
+    _request_surrounding_text (ibusimcontext);
 
     g_object_add_weak_pointer ((GObject *) context,
                                (gpointer *) &_focus_im_context);
@@ -877,9 +876,8 @@ ibus_im_context_set_client_window (GtkIMContext *context, GdkWindow *client)
 }
 
 static gboolean
-_set_cursor_location_internal (GtkIMContext *context)
+_set_cursor_location_internal (IBusIMContext *ibusimcontext)
 {
-    IBusIMContext *ibusimcontext = IBUS_IM_CONTEXT (context);
     GdkRectangle area;
 
     if(ibusimcontext->client_window == NULL ||
@@ -925,7 +923,7 @@ ibus_im_context_set_cursor_location (GtkIMContext *context, GdkRectangle *area)
         return;
     }
     ibusimcontext->cursor_area = *area;
-    _set_cursor_location_internal (context);
+    _set_cursor_location_internal (ibusimcontext);
     gtk_im_context_set_cursor_location (ibusimcontext->slave, area);
 }
 
@@ -936,17 +934,79 @@ ibus_im_context_set_use_preedit (GtkIMContext *context, gboolean use_preedit)
 
     IBusIMContext *ibusimcontext = IBUS_IM_CONTEXT (context);
 
+    if (use_preedit) {
+        ibusimcontext->caps |= IBUS_CAP_PREEDIT_TEXT;
+    }
+    else {
+        ibusimcontext->caps &= ~IBUS_CAP_PREEDIT_TEXT;
+    }
     if(ibusimcontext->ibuscontext) {
-        if (use_preedit) {
-            ibusimcontext->caps |= IBUS_CAP_PREEDIT_TEXT;
-        }
-        else {
-            ibusimcontext->caps &= ~IBUS_CAP_PREEDIT_TEXT;
-        }
+        ibus_input_context_set_capabilities (ibusimcontext->ibuscontext,
+                                             ibusimcontext->caps);
     }
     gtk_im_context_set_use_preedit (ibusimcontext->slave, use_preedit);
 }
 
+static guint
+get_selection_anchor_point (IBusIMContext *ibusimcontext,
+                            guint cursor_pos,
+                            guint surrounding_text_len)
+{
+    GtkWidget *widget;
+    if (ibusimcontext->client_window == NULL) {
+        return cursor_pos;
+    }
+    gdk_window_get_user_data (ibusimcontext->client_window, (gpointer *)&widget);
+
+    if (!GTK_IS_TEXT_VIEW (widget)){
+        return cursor_pos;
+    }
+
+    GtkTextView *text_view = GTK_TEXT_VIEW (widget);
+    GtkTextBuffer *buffer = gtk_text_view_get_buffer (text_view);
+
+    if (!gtk_text_buffer_get_has_selection (buffer)) {
+        return cursor_pos;
+    }
+
+    GtkTextIter start_iter, end_iter, cursor_iter;
+    if (!gtk_text_buffer_get_selection_bounds (buffer, &start_iter, &end_iter)) {
+        return cursor_pos;
+    }
+
+    gtk_text_buffer_get_iter_at_mark (buffer,
+                                      &cursor_iter,
+                                      gtk_text_buffer_get_insert (buffer));
+
+    guint start_index = gtk_text_iter_get_offset (&start_iter);
+    guint end_index   = gtk_text_iter_get_offset (&end_iter);
+    guint cursor_index = gtk_text_iter_get_offset (&cursor_iter);
+
+    guint anchor;
+
+    if (start_index == cursor_index) {
+      anchor = end_index;
+    } else if (end_index == cursor_index) {
+      anchor = start_index;
+    } else {
+      return cursor_pos;
+    }
+
+    // Change absolute index to relative position.
+    guint relative_origin = cursor_index - cursor_pos;
+
+    if (anchor < relative_origin) {
+      return cursor_pos;
+    }
+    anchor -= relative_origin;
+
+    if (anchor > surrounding_text_len) {
+      return cursor_pos;
+    }
+
+    return anchor;
+}
+
 static void
 ibus_im_context_set_surrounding (GtkIMContext  *context,
                                  const gchar   *text,
@@ -964,15 +1024,22 @@ ibus_im_context_set_surrounding (GtkIMContext  *context,
     if (ibusimcontext->enable && ibusimcontext->ibuscontext) {
         IBusText *ibustext;
         guint cursor_pos;
+        guint utf8_len;
         gchar *p;
 
         p = g_strndup (text, len);
         cursor_pos = g_utf8_strlen (p, cursor_index);
+        utf8_len = g_utf8_strlen(p, len);
         ibustext = ibus_text_new_from_string (p);
         g_free (p);
+
+        guint anchor_pos = get_selection_anchor_point (ibusimcontext,
+                                                       cursor_pos,
+                                                       utf8_len);
         ibus_input_context_set_surrounding_text (ibusimcontext->ibuscontext,
                                                  ibustext,
-                                                 cursor_pos);
+                                                 cursor_pos,
+                                                 anchor_pos);
     }
     gtk_im_context_set_surrounding (ibusimcontext->slave,
                                     text,
@@ -1000,7 +1067,7 @@ _ibus_context_commit_text_cb (IBusInputContext *ibuscontext,
 
     g_signal_emit (ibusimcontext, _signal_commit_id, 0, text->text);
 
-    _request_surrounding_text (ibusimcontext, FALSE);
+    _request_surrounding_text (ibusimcontext);
 }
 
 static gboolean
@@ -1296,7 +1363,7 @@ _ibus_context_show_preedit_text_cb (IBusInputContext   *ibuscontext,
     g_signal_emit (ibusimcontext, _signal_preedit_start_id, 0);
     g_signal_emit (ibusimcontext, _signal_preedit_changed_id, 0);
 
-    _request_surrounding_text (ibusimcontext, FALSE);
+    _request_surrounding_text (ibusimcontext);
 }
 
 static void
@@ -1323,7 +1390,7 @@ _ibus_context_enabled_cb (IBusInputContext *ibuscontext,
 
     /* retrieve the initial surrounding-text (regardless of whether
      * the current IBus engine needs surrounding-text) */
-    _request_surrounding_text (ibusimcontext, TRUE);
+    _request_surrounding_text (ibusimcontext);
 }
 
 static void
@@ -1427,6 +1494,7 @@ _create_input_context_done (IBusBus       *bus,
 
         if (ibusimcontext->has_focus) {
             ibus_input_context_focus_in (ibusimcontext->ibuscontext);
+            _set_cursor_location_internal (ibusimcontext);
         }
     }
 
